@@ -71,7 +71,7 @@ gdk_gc_directfb_get_type (void)
 
   if (!object_type)
     {
-      static const GTypeInfo object_info =
+      const GTypeInfo object_info =
       {
         sizeof (GdkGCDirectFBClass),
         (GBaseInitFunc) NULL,
