@@ -83,11 +83,13 @@
 *tp++ = gtk_accessible_get_type();
 *tp++ = gtk_action_get_type();
 *tp++ = gtk_action_group_get_type();
+*tp++ = gtk_activatable_get_type();
 *tp++ = gtk_adjustment_get_type();
 *tp++ = gtk_alignment_get_type();
 *tp++ = gtk_anchor_type_get_type();
 *tp++ = gtk_arg_flags_get_type();
 *tp++ = gtk_arrow_get_type();
+*tp++ = gtk_arrow_placement_get_type();
 *tp++ = gtk_arrow_type_get_type();
 *tp++ = gtk_aspect_frame_get_type();
 *tp++ = gtk_assistant_get_type();
@@ -116,6 +118,7 @@
 *tp++ = gtk_cell_renderer_pixbuf_get_type();
 *tp++ = gtk_cell_renderer_progress_get_type();
 *tp++ = gtk_cell_renderer_spin_get_type();
+*tp++ = gtk_cell_renderer_spinner_get_type();
 *tp++ = gtk_cell_renderer_state_get_type();
 *tp++ = gtk_cell_renderer_text_get_type();
 *tp++ = gtk_cell_renderer_toggle_get_type();
@@ -136,7 +139,6 @@
 *tp++ = gtk_ctree_expansion_type_get_type();
 *tp++ = gtk_ctree_line_style_get_type();
 *tp++ = gtk_ctree_pos_get_type();
-*tp++ = gtk_curve_get_type();
 *tp++ = gtk_curve_type_get_type();
 *tp++ = gtk_debug_flag_get_type();
 *tp++ = gtk_delete_type_get_type();
@@ -147,8 +149,10 @@
 *tp++ = gtk_drag_result_get_type();
 *tp++ = gtk_drawing_area_get_type();
 *tp++ = gtk_editable_get_type();
+*tp++ = gtk_entry_buffer_get_type();
 *tp++ = gtk_entry_completion_get_type();
 *tp++ = gtk_entry_get_type();
+*tp++ = gtk_entry_icon_position_get_type();
 *tp++ = gtk_event_box_get_type();
 *tp++ = gtk_expander_get_type();
 *tp++ = gtk_expander_style_get_type();
@@ -166,7 +170,6 @@
 *tp++ = gtk_font_selection_dialog_get_type();
 *tp++ = gtk_font_selection_get_type();
 *tp++ = gtk_frame_get_type();
-*tp++ = gtk_gamma_curve_get_type();
 *tp++ = gtk_handle_box_get_type();
 *tp++ = gtk_hbox_get_type();
 *tp++ = gtk_hbutton_box_get_type();
@@ -175,6 +178,7 @@
 *tp++ = gtk_hscale_get_type();
 *tp++ = gtk_hscrollbar_get_type();
 *tp++ = gtk_hseparator_get_type();
+*tp++ = gtk_hsv_get_type();
 *tp++ = gtk_icon_factory_get_type();
 *tp++ = gtk_icon_info_get_type();
 *tp++ = gtk_icon_lookup_flags_get_type();
@@ -194,7 +198,7 @@
 *tp++ = gtk_im_multicontext_get_type();
 *tp++ = gtk_im_preedit_style_get_type();
 *tp++ = gtk_im_status_style_get_type();
-*tp++ = gtk_input_dialog_get_type();
+*tp++ = gtk_info_bar_get_type();
 *tp++ = gtk_invisible_get_type();
 *tp++ = gtk_item_get_type();
 *tp++ = gtk_justification_get_type();
@@ -217,8 +221,11 @@
 *tp++ = gtk_movement_step_get_type();
 *tp++ = gtk_notebook_get_type();
 *tp++ = gtk_notebook_tab_get_type();
+*tp++ = gtk_number_up_layout_get_type();
 *tp++ = gtk_object_flags_get_type();
 *tp++ = gtk_object_get_type();
+*tp++ = gtk_offscreen_window_get_type();
+*tp++ = gtk_orientable_get_type();
 *tp++ = gtk_orientation_get_type();
 *tp++ = gtk_pack_direction_get_type();
 *tp++ = gtk_pack_type_get_type();
@@ -295,6 +302,7 @@
 *tp++ = gtk_sort_type_get_type();
 *tp++ = gtk_spin_button_get_type();
 *tp++ = gtk_spin_button_update_policy_get_type();
+*tp++ = gtk_spinner_get_type();
 *tp++ = gtk_spin_type_get_type();
 *tp++ = gtk_state_type_get_type();
 *tp++ = gtk_statusbar_get_type();
@@ -328,6 +336,9 @@
 *tp++ = gtk_toolbar_style_get_type();
 *tp++ = gtk_tool_button_get_type();
 *tp++ = gtk_tool_item_get_type();
+*tp++ = gtk_tool_item_group_get_type();
+*tp++ = gtk_tool_palette_drag_targets_get_type();
+*tp++ = gtk_tool_palette_get_type();
 *tp++ = gtk_tool_shell_get_type();
 *tp++ = gtk_tooltip_get_type();
 *tp++ = gtk_tree_drag_dest_get_type();
